package main

import (
	"os"
	"os/user"
	"path"

	"github.com/jessevdk/go-flags"
	"gopkg.in/yaml.v3"
)

const exitCodeParseArgumentError = 2

type configPathsStruct struct {
	current string
	user    string
	main    string
}

var (
	configPaths = configPathsStruct{
		main: "/etc/requirements-manager/config.yaml",
	}
)

const defaultTemplate = `
## Overview

Created: {{ .created }}

{{.Meta}}

## Tasks

{{.Tasks}}

## Items

{{.Items}}

## requirements

{{.Requirements}}


`

// Exporting interface instead of struct
//type Configuration interface{}

// Configuration
type Configuration struct {
	Path string `yaml:"path"`

	Locale struct {
		DateFormat string `yaml:"date-format"`
	} `yaml:"Locale"`

	Table struct {
		Columns []string `yaml:"columns"`
	} `yaml:"table"`

	Overview struct {
		Template string `yaml:"template"`
	} `yaml:"overview"`
}

func fileExists(name string) (found bool) {
	if f, err := os.Stat(name); err == nil {
		return f.Mode().IsRegular()
	}
	return false
}

type commandLineOptions struct {
	Path       string `short:"p" long:"path" description:"define the path where the specifications are located" default:"."`
	DateFormat string `short:"f" long:"date-format" description:"date format" default:"2006-01-02"`
	Meta       struct {
		Print struct {
			Columns []string `short:"c" long:"column" description:"defines the columns used"`
		} `command:"print"`
	} `command:"meta"`
	ToDos struct {
		Print struct {
		} `command:"print"`
	} `command:"todos"`
	Items struct {
		Print struct {
		} `command:"print"`
	} `command:"items"`
	Overview struct {
		Print struct {
			Columns      []string `short:"c" long:"column" description:"defines the columns used"`
			TemplatePath string   `short:"t" long:"template" description:"file name of the template of the overview page"`
			Output       string   `short:"o" long:"output" description:"redirect output to a file"`
			Format       string   `short:"f" long:"format" description:"the desired output format (pdf, md)"`
		} `command:"print"`
	} `command:"overview"`
	Version struct {
	} `command:"version"`
}

// NewConfiguration
func NewConfiguration() (*flags.Command, *commandLineOptions, *Configuration) {

	arguments := new(commandLineOptions)
	p := flags.NewParser(arguments, flags.Default)
	_, err := p.Parse()
	if err != nil {
		os.Exit(-1)
	}

	current, err := os.Getwd()
	if err != nil {
		panic(err)
	}

	filename := path.Clean(current + "/config.yaml")

	cfg := Configuration{}

	usr, err := user.Current()
	if err == nil {
		configPaths.user = usr.HomeDir + "/.config/requirements-manager/config.yaml"
	}

	switch {
	case fileExists(filename):
		readFile(&cfg, filename)
	case err == nil && fileExists(current):
		readFile(&cfg, current)
	case fileExists(configPaths.user):
		readFile(&cfg, configPaths.user)
	case fileExists(configPaths.main):
		readFile(&cfg, configPaths.main)
	}

	//readEnv(&cfg)

	if arguments.Path != "" {
		cfg.Path = arguments.Path
	}

	if arguments.DateFormat != "" {
		cfg.Locale.DateFormat = arguments.DateFormat
	}

	activeCommand := p.Command.Active

	switch activeCommand.Name {

	case "version":

	case "overview":
		subcommand := activeCommand.Active
		switch subcommand.Name {
		case "print":

			if len(arguments.Overview.Print.Columns) > 0 {
				cfg.Table.Columns = validateColumns(arguments.Overview.Print.Columns)
			}

			if arguments.Overview.Print.TemplatePath != "" {
				template, err := os.ReadFile(arguments.Overview.Print.TemplatePath)
				if err != nil {
					printErrorAndExit(exitCodeCatchAll,
						"the specified template", arguments.Overview.Print.TemplatePath)
				}
				cfg.Overview.Template = string(template)
			} else {
				cfg.Overview.Template = defaultTemplate
			}
		}
	case "meta":
		subcommand := activeCommand.Active
		switch subcommand.Name {
		case "print":
			if len(arguments.Meta.Print.Columns) > 0 {
				cfg.Table.Columns = validateColumns(arguments.Meta.Print.Columns)
			}
		}
	}

	return p.Command.Active, arguments, &cfg
}

func validateColumns(columns []string) []string {

	//for i, column := range columns {
	//	columns[i] = printer.Sprintf(column)
	//}

	// VALIDATE COLUMS NAME

	//
	//for _, name := range config.Meta.Columns {
	//
	//	field := reflect.Indirect(r).FieldByName(name)
	//	if field == (reflect.Value{}) {
	//		printError("the specified column " + name + " for the table does not exist")
	//		continue

	//	}

	return columns

}

func readFile(cfg *Configuration, filename string) {
	f, err := os.Open(filename)
	if err != nil {
		printErrorAndExit(2, err.Error())
	}

	defer f.Close()

	decoder := yaml.NewDecoder(f)
	err = decoder.Decode(cfg)
	if err != nil {
		printErrorAndExit(2, err.Error())
	}
}
